#ifndef PgpSdk8CallsH
#define PgpSdk8CallsH

#include "pgpKeys.h"
#include "pgpErrors.h"
#include "PgpEncode.h"
#include "pgpUtilities.h"
#include "pgpFeatures.h"
#include "pgpOptionList.h"
#include "pgpMemoryMgr.h"
#include "pgpRandomPool.h"

// only functions with option lists need prototypes
PGPOptionListRef PGPOConventionalEncrypt(PGPContextRef Context,
      PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3);
PGPError PGPEncode(PGPContextRef context, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6, PGPOptionListRef opt7,
   PGPOptionListRef opt8, PGPOptionListRef opt9, PGPOptionListRef opt10,
   PGPOptionListRef opt11, PGPOptionListRef opt12, PGPOptionListRef opt13,
   PGPOptionListRef opt14, PGPOptionListRef opt15,
   PGPOptionListRef opt16);
PGPError PGPDecode(PGPContextRef context, PGPOptionListRef opt1, PGPOptionListRef opt2,
   PGPOptionListRef opt3, PGPOptionListRef opt4, PGPOptionListRef opt5,
   PGPOptionListRef opt6, PGPOptionListRef opt7, PGPOptionListRef opt8,
   PGPOptionListRef opt9);
PGPError PGPExport(PGPContextRef context, PGPOptionListRef opt1, PGPOptionListRef opt2,
   PGPOptionListRef opt3, PGPOptionListRef opt4,PGPOptionListRef opt5,
   PGPOptionListRef opt6);
PGPError PGPImport(PGPContextRef context, PGPKeyDBRef *keyDB, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6);
PGPBoolean PGPPassphraseIsValid(PGPKeyDBObjRef key, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4);
PGPOptionListRef PGPOSignWithKey(PGPContextRef context, PGPKeyDBObjRef sigKey,
   PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3,
   PGPOptionListRef opt4);
PGPError PGPAppendOptionList(PGPOptionListRef outList, PGPOptionListRef opt1,
   PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
   PGPOptionListRef opt5, PGPOptionListRef opt6,
   PGPOptionListRef opt7, PGPOptionListRef opt8);

// callback
PGPError DecodeEventHandler(PGPContextRef context, PGPEvent *event, PGPUserValue userValue);

// utility
bool __fastcall IsPgpError(PGPError err, const AnsiString &errmsg);
AnsiString __fastcall TimeT2SigTime(time_t thetime);

// typedefs for sdk funtions
typedef PGPUInt32 (*PGPSDKINITV8)(PGPFlags);
typedef PGPError  (*PGPSDKCLEANUP)();
typedef PGPUInt32 (*PGPGETPGPSDKVERSION)();
typedef PGPUInt32 (*PGPGETPGPSDKAPIVERSION)();
typedef PGPError  (*PGPOPENKEYDBFILE)(PGPContextRef, PGPOpenKeyDBFileOptions,
   PGPFileSpecRef, PGPFileSpecRef, PGPKeyDBRef*);
typedef PGPError  (*PGPNEWCONTEXT)(PGPUInt32, PGPContextRef*);
typedef PGPError (*PGPFREECONTEXT)(PGPContextRef*);
typedef PGPError (*PGPNEWKEYSET)(PGPKeyDBRef, PGPKeySetRef*);
typedef PGPError (*PGPNEWEMPTYKEYSET)(PGPKeyDBRef, PGPKeySetRef*);
typedef PGPError (*PGPGETERRORSTRING)(PGPError, PGPSize, char*);

typedef PGPError (*PGPGETFULLPATHFROMFILESPEC)(PGPFileSpecRef, char**);
typedef PGPError (*PGPFREEFILESPEC)(PGPFileSpecRef fileRef);

typedef PGPError (*PGPFREEKEYSET)(PGPKeySetRef);
typedef PGPError (*PGPFREEDATA)(void *allocation);

typedef PGPError (*PGPNEWKEYDBOBJDATAFILTER)(PGPContextRef, PGPKeyDBObjProperty,
   const void*, PGPSize, PGPMatchCriterion, PGPFilterRef*);
typedef PGPError (*PGPNEWKEYDISABLEDFILTER)(PGPContextRef, PGPBoolean, PGPFilterRef*);
typedef PGPError (*PGPNEWKEYREVOKEDFILTER)(PGPContextRef, PGPBoolean, PGPFilterRef*);
typedef PGPError (*PGPFILTERKEYSET)(PGPKeySetRef, PGPFilterRef, PGPKeySetRef*);
typedef PGPError (*PGPORDERKEYSET)(PGPKeySetRef, PGPKeyOrdering, PGPBoolean, PGPKeyListRef*);
typedef PGPError (*PGPNEWKEYITER)(PGPKeyListRef, PGPKeyIterRef*);
typedef PGPError (*PGPGETKEYID)(PGPKeyDBObjRef, PGPKeyID*);
typedef PGPError (*PGPGETKEYIDSTRING)(PGPKeyID const*, PGPKeyIDStringType,
   char[kPGPMaxKeyIDStringSize]);
typedef PGPError (*PGPFREEKEYITER)(PGPKeyIterRef);
typedef PGPError (*PGPFREEKEYLIST)(PGPKeyListRef);
typedef PGPError (*PGPFREEFILTER)(PGPFilterRef);
typedef PGPError (*PGPNEWFILESPECFROMFULLPATH)(PGPContextRef, const char*, PGPFileSpecRef*);
typedef PGPError (*PGPDECODE)(PGPContextRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPENCODE)(PGPContextRef context, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPADDKEYS)(PGPKeySetRef, PGPKeySetRef);
typedef PGPError (*PGPCOMMITKEYRINGCHANGES)(PGPKeySetRef);
// options
typedef PGPError (*PGPNEWOPTIONLIST)(PGPContextRef, PGPOptionListRef*);
typedef PGPError (*PGPAPPENDOPTIONLIST)(PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPFREEOPTIONLIST)(PGPOptionListRef);
typedef PGPOptionListRef (*PGPOEXPORTFORMAT)(PGPContextRef, PGPExportFormat);
typedef PGPOptionListRef (*PGPOEXPORTPRIVATEKEYS)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOVERSIONSTRING)(PGPContextRef, const char*);
typedef PGPOptionListRef (*PGPOLASTOPTION)(PGPContextRef);
typedef PGPOptionListRef (*PGPOALLOCATEDOUTPUTBUFFER)(PGPContextRef, void**, PGPSize,
   PGPSize*);
typedef PGPOptionListRef (*PGPOINPUTBUFFER)(PGPContextRef, void const*, PGPSize);
typedef PGPOptionListRef (*PGPOKEYSETREF)(PGPContextRef context, PGPKeySetRef keysetRef);
typedef PGPOptionListRef (*PGPOEVENTHANDLER)(PGPContextRef, PGPEventHandlerProcPtr,
   PGPUserValue);
typedef PGPOptionListRef (*PGPORECURSIVELYDECODE)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOPASSPHRASE)(PGPContextRef, const char*);
typedef PGPOptionListRef (*PGPOCLEARSIGN)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOCOMPRESSION)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOARMOROUTPUT)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOENCRYPTTOKEYSET)(PGPContextRef,PGPKeySetRef);
typedef PGPOptionListRef (*PGPOCONVENTIONALENCRYPT)(PGPContextRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPOptionListRef (*PGPOOUTPUTLINEENDTYPE)(PGPContextRef,PGPLineEndType);
typedef PGPOptionListRef (*PGPODDATAISSCII)(PGPContextRef,PGPBoolean);
typedef PGPOptionListRef (*PGPOFORYOUREYESONLY)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOHASHALGORITHM)(PGPContextRef, PGPHashAlgorithm);
typedef PGPOptionListRef (*PGPOEXPORTKEYSET)(PGPContextRef, PGPKeySetRef);
typedef PGPOptionListRef (*PGPOKEYDBREF)(PGPContextRef, PGPKeyDBRef);
typedef PGPOptionListRef (*PGPOSIGNWITHKEY)(PGPContextRef, PGPKeyDBObjRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPOptionListRef (*PGPOINPUTFILE)(PGPContextRef, PGPFileSpecRef);
typedef PGPOptionListRef (*PGPOASKUSERFORENTROPY)(PGPContextRef, PGPBoolean);

typedef PGPError (*PGPNEWKEYITERFROMKEYDB)(PGPKeyDBRef, PGPKeyIterRef*);
typedef PGPError (*PGPKEYITERNEXTKEYDBOBJ)(PGPKeyIterRef, PGPKeyDBObjType, PGPKeyDBObjRef*);
typedef PGPError (*PGPNEWKEYDBOBJBOOLEANFILTER)(PGPContextRef, PGPKeyDBObjProperty,
   PGPBoolean, PGPFilterRef*);
typedef PGPError (*PGPEXPORT)(PGPContextRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPIMPORT)(PGPContextRef, PGPKeyDBRef*, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPFILTERKEYDB)(PGPKeyDBRef keyDB, PGPFilterRef filter,
   PGPKeySetRef *resultSet);
typedef PGPError (*PGPFREEKEYDB)(PGPKeyDBRef);
typedef PGPError (*PGPNEWKEYIDFROMSTRING)(const char*, PGPPublicKeyAlgorithm, PGPKeyID*);
typedef PGPError (*PGPFINDKEYBYKEYID)(PGPKeyDBRef, const PGPKeyID*, PGPKeyDBObjRef*);
typedef PGPBoolean (*PGPPASSPHRASEISVALID)(PGPKeyDBObjRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, ...);
typedef PGPError (*PGPGETKEYDBOBJNUMERICPROPERTY)(PGPKeyDBObjRef key,
   PGPKeyDBObjProperty whichProperty, PGPInt32 *prop);
typedef PGPError (*PGPGETKEYOBJNUMERICPROPERTY)(PGPKeyDBObjRef, PGPKeyDBObjProperty, PGPInt32*);
typedef PGPError (*PGPGETKEYDBOBJTIMEPROPERTY)(PGPKeyDBObjRef, PGPKeyDBObjProperty, PGPTime*);
typedef PGPError (*PGPGETKEYDBOBJBOOLEANPROPERTY)(PGPKeyDBObjRef, PGPKeyDBObjProperty,PGPBoolean*);
typedef PGPError (*PGPCOUNTADDITIONALRECIPIENTREQUESTS)(PGPKeyDBObjRef, PGPUInt32*);
typedef PGPError (*PGPADDKEY)(PGPKeyDBObjRef, PGPKeySetRef);
typedef PGPError (*PGPNEWKEYITERFROMKEYSET)(PGPKeySetRef keySet, PGPKeyIterRef *iter);
typedef PGPError (*PGPGETPRIMARYUSERIDNAME)(PGPKeyDBObjRef, void*, PGPSize, PGPSize*);
typedef PGPError (*PGPCOPYKEYS)(PGPKeySetRef, PGPKeyDBRef, PGPKeySetRef*);
typedef PGPError (*PGPNEWONEKEYSET)(PGPKeyDBObjRef, PGPKeySetRef*);
typedef PGPError (*PGPGETKEYIDALGORITHM)(const PGPKeyID*, PGPPublicKeyAlgorithm*);
typedef PGPUInt32 (*PGPGLOBALRANDOMPOOLMOUSEMOVED)();
typedef PGPError (*PGPCONTEXTGETRANDOMBYTES)(PGPContextRef, void*, PGPSize);
typedef time_t (*PGPGETSTDTIMEFROMPGPTIME)(PGPTime);
typedef PGPTime (*PGPGETTIME)();

struct Tdecinfo8 {
   PGPKeyDBRef keyDB;
   TStrings *siginfos;
   int decrypts;
   int sigs;
};

extern HANDLE hSdkLib8;
extern PGPContextRef Context8;
extern PGPError err;
//---------------------------------------------------------------------------
#endif
